@@ -1,9 +1,11 @@
 //! This module defines various traits required by the users of the library to implement.
+use bellperson::{gadgets::num::AllocatedNum, ConstraintSystem, SynthesisError};
 use core::borrow::Borrow;
 use core::fmt::Debug;
 use core::ops::{Add, AddAssign, Mul, MulAssign, Neg, Sub, SubAssign};
 use merlin::Transcript;
 use rand::{CryptoRng, RngCore};
+use rug::Integer;
 
 /// Represents an element of a prime field
 pub trait PrimeField:
@@ -40,6 +42,9 @@ pub trait PrimeField:
 
   /// returns an uniformly random element from the finite field
   fn random(rng: &mut (impl RngCore + CryptoRng)) -> Self;
+
+  /// Get prime field order as a rug::Integer
+  fn get_order() -> Integer;
 }
 
 /// Represents an element of a group
@@ -54,6 +59,9 @@ pub trait Group:
   + ScalarMul<<Self as Group>::Scalar>
   + ScalarMulOwned<<Self as Group>::Scalar>
 {
+  /// A type representing an element of the base field of the group
+  type Base: PrimeField;
+
   /// A type representing an element of the scalar field of the group
   type Scalar: PrimeField + ChallengeTrait;
 
@@ -75,6 +83,9 @@ pub trait Group:
   /// Attempts to create a group element from a sequence of bytes,
   /// failing with a `None` if the supplied bytes do not encode the group element
   fn from_uniform_bytes(bytes: &[u8]) -> Option<Self>;
+
+  /// Returns the affine coordinates (x, y, infinty) for the point
+  fn to_coordinates(&self) -> (Self::Base, Self::Base, bool);
 }
 
 /// Represents a compressed version of a group element
@@ -119,3 +130,13 @@ impl<T, Rhs, Output> ScalarMul<Rhs, Output> for T where T: Mul<Rhs, Output = Out
 /// A helper trait for references implementing group scalar multiplication.
 pub trait ScalarMulOwned<Rhs, Output = Self>: for<'r> ScalarMul<&'r Rhs, Output> {}
 impl<T, Rhs, Output> ScalarMulOwned<Rhs, Output> for T where T: for<'r> ScalarMul<&'r Rhs, Output> {}
+
+///A helper trait for the inner circuit F
+pub trait InnerCircuit<F: PrimeField + ff::PrimeField> {
+  ///Sythesize the circuit for a computation step and return variable that corresponds to z_{i+1}
+  fn synthesize<CS: ConstraintSystem<F>>(
+    &self,
+    cs: &mut CS,
+    z: AllocatedNum<F>,
+  ) -> Result<AllocatedNum<F>, SynthesisError>;
+}
